/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.ajax;

import javax.faces.component.ActionSource;
import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.ActionEvent;
import javax.faces.event.ActionListener;
import javax.faces.event.FacesEvent;
import javax.faces.event.PhaseId;

import org.ajax4jsf.framework.renderer.AjaxRendererUtils;


/**
 * @author shura (latest modification by $Author: alexsmirnov $)
 * @version $Revision: 1.3 $ $Date: 2006/12/07 19:45:26 $
 *
 */
public abstract class AjaxActionComponent extends UIComponentBase implements ActionSource,
		AjaxComponent, AjaxSource {
	
	/** 
     * @exception NullPointerException 
     */ 
    public void addActionListener(ActionListener listener) {

        addFacesListener(listener);

    }


    public ActionListener[] getActionListeners() {

        ActionListener al[] = (ActionListener [])
	    getFacesListeners(ActionListener.class);
        return (al);

    }



    /**
     * @exception NullPointerException 
     */ 
    public void removeActionListener(ActionListener listener) {

        removeFacesListener(listener);

    }


	/* (non-Javadoc)
	 * @see javax.faces.component.UIComponentBase#broadcast(javax.faces.event.FacesEvent)
	 */
	public void broadcast(FacesEvent event) throws AbortProcessingException {
		// perform default
		super.broadcast(event);
		if (event instanceof ActionEvent) {
            FacesContext context = getFacesContext();
            // Perform actionListener.
            MethodBinding action = getActionListener();
            if (null != action) {
				action.invoke(context, new Object[] { event });
			}
            action = getAction();
            // Difference from command - default action listener not perform for null actions ( ajax not use navigations, in common ).
            // TODO - make navigation to different page ?
            if(null != action){
                ActionListener listener =
                    context.getApplication().getActionListener();
                  if (listener != null) {
                      listener.processAction((ActionEvent) event);
                  }
            	
            } else {
				context.renderResponse();
			}
		} else if (event instanceof AjaxEvent) {
			// complete re-Render fields. AjaxEvent deliver before render response.
			setupReRender();
		}
	}


	/**
	 * Template methods for fill set of resions to render in subclasses.
	 */
	protected void setupReRender() {
		AjaxRendererUtils.addRegionsFromComponent(this,getFacesContext());
		
	}


	/* (non-Javadoc)
	 * @see javax.faces.component.UIComponentBase#queueEvent(javax.faces.event.FacesEvent)
	 */
	public void queueEvent(FacesEvent event) {
		if (event instanceof ActionEvent && event.getComponent()==this) {
			if (isImmediate()) {
				event.setPhaseId(PhaseId.APPLY_REQUEST_VALUES);
			}  else if (isBypassUpdates()) {
				event.setPhaseId(PhaseId.PROCESS_VALIDATIONS);
			}else {
				event.setPhaseId(PhaseId.INVOKE_APPLICATION);
			}
		}
		super.queueEvent(event);
	}


	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.ajax.AjaxSource#addAjaxListener(org.ajax4jsf.framework.ajax.AjaxListener)
	 */
	public void addAjaxListener(AjaxListener listener) {
        addFacesListener(listener);		
	}


	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.ajax.AjaxSource#getAjaxListeners()
	 */
	public AjaxListener[] getAjaxListeners() {
        AjaxListener al[] = (AjaxListener [])
	    getFacesListeners(AjaxListener.class);
        return (al);

	}


	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.ajax.AjaxSource#removeAjaxListener(org.ajax4jsf.framework.ajax.AjaxListener)
	 */
	public void removeAjaxListener(AjaxListener listener) {
		removeFacesListener(listener);
		
	}

    
    


}
