/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.ajax;


/**
 * Base Interface for Ajax-enabled acting components.
 * TODO - extend <code>ActionSource</code>???
 * @author shura (latest modification by $Author: alexsmirnov $)
 * @version $Revision: 1.4 $ $Date: 2006/11/24 17:10:22 $
 *
 */
public interface AjaxComponent
{
	
	public static final String AJAX_COMPONETT_PARAMETER = AjaxComponent.class.getName()+".parameter";

    /**
     * setter method for property
     * @param new value of String, <code>Collection</code> or array of component's Id , updated in case of Ajax request by parent component. to set
     */
    public abstract void setReRender(Object targetId);

    /**
     * @return value or result of valueBinding of String, <code>Collection</code> or array of component's Id , updated in case of Ajax request by parent component.
     */
    public abstract Object getReRender();

    /**
     * setter method for property
     * @param new value of ajaxType of control component - link or input to set
     */
//    public abstract void setAjaxType(String ajaxType);

    /**
     * @return value or result of valueBinding of ajaxType of control component - link or input
     */
//    public abstract String getAjaxType();

    /**
     * setter method for property
     * @param new value of Id ( in format of UIComponent.findComponent() call ) of request status indicator  to set
     */
    public abstract void setStatus(String status);

    /**
     * @return value or result of valueBinding of Id ( in format of UIComponent.findComponent() call ) of request status indicator 
     */
    public abstract String getStatus();

    /**
     * setter method for property
     * @param new value of Name of JavaScript function, called on complete Ajax request to set
     */
    public abstract void setOncomplete(String oncomplete);

    /**
     * @return value or result of valueBinding of Name of JavaScript function, called on complete Ajax request
     */
    public abstract String getOncomplete();

    /**
     * setter method for property
     * @param new value of Submit ( or not ) full form on Ajax action. to set
     */
    public abstract void setLimitToList(boolean submitForm);

    /**
     * @return value or result of valueBinding of Submit ( or not ) full form on Ajax action.
     */
    public abstract boolean isLimitToList();

    /**
     * setter method for property
     * @param new value of Submit ( or not ) full form on Ajax action. to set
     */
    public abstract void setAjaxSingle(boolean single);

    /**
     * @return value or result of valueBinding of Submit ( or not ) full form on Ajax action.
     */
    public abstract boolean isAjaxSingle();
	
    /**
     * Getter for bypassUpdates bean property. Indicate that component must invoke listeners after Process Validators phase
     * and force render response after it - since no values of components will be updated. Can be used to perform validation of client input
     * using server-side validators.
     * @return true if component must force render after validation phase.
     */
    public abstract boolean isBypassUpdates();
    
    /**
     * Setter for bypassUpdates bean property.
     * @param bypass true if component must force render after validation phase.
     */
    public abstract void setBypassUpdates(boolean bypass);
    
    /**
	 * Getter for property name of events queue on client side - for avoid to send requests too frequnly ( on key events, for example ), implementation will be generated by
	 * componnents-generator
	 * @return property value
	 */
	public abstract String getEventsQueue();

	/**
	 * Setter for flag indicated aborting unfinished ajax requests in queue. 
	 * @param newvalue - new property value. If true, unfinished request in queue will be aborted on new events.
	 */
	public abstract void setIgnoreDupResponses(boolean newvalue);
	
	
    /**
	 * Getter for property flag indicated aborting unfinished ajax requests in queue.
	 * componnents-generator
	 * @return property value
	 */
	public abstract boolean isIgnoreDupResponses();

	/**
	 * Setter for property name of events queue on client side - for avoid to send requests too frequnly ( on key events, for example ), implementation will be generated by
	 * componnents-generator
	 * @param newvalue - new property value
	 */
	public abstract void setEventsQueue(String newvalue);
	
	/**
	 * Getter for property Delay ( in ms. ) for send ajax request on JavaScript event. In conjunction with events queue can reduce number of requests on keyboard or mouse move events., implementation will be generated by
	 * componnents-generator
	 * @return property value
	 */
	public abstract int getRequestDelay();

	/**
	 * Setter for property Delay ( in ms. ) for send ajax request on JavaScript event. In conjunction with events queue can reduce number of requests on keyboard or mouse move events., implementation will be generated by
	 * componnents-generator
	 * @param newvalue - new property value
	 */
	public abstract void setRequestDelay(int newvalue);
	
	/**
	 * Getter for request timeout 
	 * @return timeout in ms.
	 */
	public abstract int getTimeout();
	
	/**
	 * Setter for request timeout
	 * @param timeout new value in ms.
	 */
	public abstract void setTimeout(int timeout);

}