/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.ajax.xmlfilter;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Date;

import javax.faces.application.ViewHandler;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.ajax4jsf.framework.renderer.AjaxContainerRenderer;
import org.ajax4jsf.framework.resource.InternetResourceService;
import org.ajax4jsf.framework.util.message.Messages;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Base class for request processing filters, with convert Htmp content to
 * XML for ajax requests, and serve request to application off-page resources
 * @author shura (latest modification by $Author: alexsmirnov $)
 * @version $Revision: 1.19 $ $Date: 2006/11/28 17:10:22 $
 *
 */
public abstract class BaseFilter implements Filter {

	private static final Log log = LogFactory.getLog(BaseFilter.class);
	public static final boolean DEBUG = true;
	private FilterConfig filterConfig;
	private static final String FUNCTION_NAME_PARAMETER = "function";
	private String function = "alert('Data received');JSHttpRequest.dataReady";
	private String attributesNames;
	private boolean rewriteid = false;
	private static final String REWRITEID_PARAMETER = "rewriteid";
	private static final String STYLESHEET_PARAMETER = "xsl";
	private static final String ABSOLUTE_TAGS_PARAMETER = "absolute-attributes";
//	private WebXml webXml;
//	private String xsl;
//	private Templates xslTemplates;
	/**
	 * 
	 */
	private static final long serialVersionUID = -2295534611886142935L;
	public static final String DATA_PARAMETER = "DATA";
	public static final String DEFAULT_SERVLET_PATH = "/resource";
	public static final String RENDERER_PREFIX = "/renderer";
	public static final String CACHEABLE_PREFIX = "/cache";
//	private static final Pattern rendererPattern = Pattern.compile(RENDERER_PREFIX+"/([^/]+)/([^/]+)/([^/]+)/(.*)");
//	private static final Pattern builderPattern = Pattern.compile(CACHEABLE_PREFIX+"/(.*)");
	public static final String FILTER_PERFORMED = "com.exade.vcp.Filter.done";
	public static final String RESPONSE_WRAPPER_ATTRIBUTE = "com.exade.vcp.Filter.ResponseWrapper";
	
	protected BaseXMLFilter xmlFilter = null;
	InternetResourceService resourceService = null;


	/**
	 * Initialize the filter.
	 */
	public void init(FilterConfig config) throws ServletException {
	        // Save config
	        filterConfig = config;
	        setFunction((String) nz(filterConfig.getInitParameter(FUNCTION_NAME_PARAMETER),getFunction()));
	        setAttributesNames(filterConfig.getInitParameter(ABSOLUTE_TAGS_PARAMETER));
	//        if( null != attributesNames){
	//            this.absoluteAttributes = attributesNames.split(",");
	//        }
	        xmlFilter.init(config);
//			xsl = config.getInitParameter(STYLESHEET_PARAMETER);
	        if("true".equalsIgnoreCase(filterConfig.getInitParameter(REWRITEID_PARAMETER))){
	            this.setRewriteid(true);
	        }
	
	        // TODO - make all parameters configurable ...
	        // Get Nekko parser features & properties.
	        // Configure faces variables and log.
//			renderKitFactory = (RenderKitFactory) FactoryFinder.getFactory(FactoryFinder.RENDER_KIT_FACTORY);
	//		InternetResourceBuilder.getInstance().setServletContext(getServletContext());
			// 	Log4J configuration. TODO - test for already configured logging.
				
				String prefix = filterConfig.getServletContext().getRealPath("/");
				String file = filterConfig.getInitParameter("log4j-init-file");
				// if the log4j-init-file is not set, then no point in trying
				if (file != null) {
					Log4JConfigurator log4jconfig = new Log4JConfigurator(prefix);
					log4jconfig.doConfigure(file);
				}
	        // init webXml configuration.
//	        WebXml.init(config.getServletContext(),config.getFilterName());
//	        this.webXml = WebXml.getInstance();
	    }

	/**
	 * @param httpServletRequest
	 * @throws UnsupportedEncodingException
	 */
	protected void setupRequestEncoding(HttpServletRequest httpServletRequest) throws UnsupportedEncodingException {
		String contentType = httpServletRequest.getHeader("Content-Type");
	
	    String characterEncoding = lookupCharacterEncoding(contentType);
	
	    if (characterEncoding == null) {
	        HttpSession session = httpServletRequest.getSession(false);
	
	        if (session != null) {
	            characterEncoding = (String) session.getAttribute(ViewHandler.CHARACTER_ENCODING_KEY);
	        }
	
	        if (characterEncoding != null) {
	            httpServletRequest.setCharacterEncoding(characterEncoding);
	        }
	    }
	}

	/**
	 * Detect request encoding from Content-Type header
	 * @param contentType
	 * @return - charset, if present.
	 */
	private String lookupCharacterEncoding(String contentType) {
	    String characterEncoding = null;
	
	    if (contentType != null)
	    {
	        int charsetFind = contentType.indexOf("charset=");
	        if (charsetFind != -1)
	        {
	            if (charsetFind == 0)
	            {
	                //charset at beginning of Content-Type, curious
	                characterEncoding = contentType.substring(8);
	            }
	            else
	            {
	                char charBefore = contentType.charAt(charsetFind - 1);
	                if (charBefore == ';' || Character.isWhitespace(charBefore))
	                {
	                    //Correct charset after mime type
	                    characterEncoding = contentType.substring(charsetFind + 8);
	                }
	            }
	            if (log.isDebugEnabled()) log.debug(Messages.getMessage(Messages.CONTENT_TYPE_ENCODING, characterEncoding));
	        }
	        else
	        {
	            if (log.isDebugEnabled()) log.debug(Messages.getMessage(Messages.CONTENT_TYPE_NO_ENCODING, contentType));
	        }
	    }
	    return characterEncoding;
	}

	/**
	 * @param initParameter
	 * @param function2
	 * @return
	 */
	private Object nz(Object param, Object def) {
	    return param != null?param:def;
	}

	/**
	 * Execute the filter.
	 */
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
		long startTimeMills =0;
	    // Detect case of request - normal, AJAX, AJAX - JavaScript
	    // TODO - detect first processing in filter.
	    HttpServletRequest httpServletRequest =(HttpServletRequest) request;
	    HttpServletResponse httpServletResponse = (HttpServletResponse) response;
		if (log.isDebugEnabled()) {
			startTimeMills = System.currentTimeMillis();
			log.debug(Messages.getMessage(Messages.FILTER_START_INFO, new Date(startTimeMills), httpServletRequest.getRequestURI()));
		}

	    if ( request.getAttribute(FILTER_PERFORMED) != Boolean.TRUE) {
	    	// mark - and not processing same request twice.
			request.setAttribute(FILTER_PERFORMED,Boolean.TRUE);
		    // check for resource request
			if ( ! getResourceService().serviceResource( httpServletRequest,
					httpServletResponse) ) {
				// Not request to resource - perform filtering.
				// first stage - detect/set encoding of request. Same as in Myfaces External Context.
			    setupRequestEncoding(httpServletRequest);
				// check ajax request parameter 
				// TODO - check for JSF page.
				if (true) {
					if (log.isDebugEnabled()) {
						log.debug(Messages.getMessage(Messages.FILTER_XML_OUTPUT));
					}

					// Execute the rest of the filter chain, including the JSP
					xmlFilter.doXmlFilter(chain, httpServletRequest, httpServletResponse);
				} else {
					// normal request, execute chain ...
					if (log.isDebugEnabled()) {
						log.debug(Messages.getMessage(Messages.FILTER_NO_XML_CHAIN));
					}
					chain.doFilter(request, response);

				}
			} 
		} else {
			if (log.isDebugEnabled()) {
				log.debug(Messages.getMessage(Messages.FILTER_NO_XML_CHAIN_2));
			}
			chain.doFilter(request, response);

		}
		if (log.isDebugEnabled()) {
			startTimeMills = System.currentTimeMillis()-startTimeMills;
			log.debug(Messages.getMessage(Messages.FILTER_STOP_INFO, "" + startTimeMills, httpServletRequest.getRequestURI()));
		}
	}

	/**
	 * @param request
	 * @return
	 */
	protected boolean isAjaxRequest(ServletRequest request) {
		try {
			return null != request
					.getParameter(AjaxContainerRenderer.AJAX_PARAMETER_NAME);
		} catch (Exception e) {
			// OCJ 10 - throw exception for static resources.
			return false;
		}
	}

	/**
	 * Destroy the filter.
	 */
	public void destroy() {
	}

	/**
	 * @return Returns the servletContext.
	 */
	ServletContext getServletContext() {
		return filterConfig.getServletContext();
	}

	/**
	 * @return the resourceService
	 * @throws ServletException 
	 */
	protected synchronized InternetResourceService getResourceService() throws ServletException {
		if (resourceService == null) {
			resourceService = new InternetResourceService();
	        // Caching initialization.
			resourceService.init(filterConfig);
			
		}
		return resourceService;
	}

	/**
	 * @param function The function to set.
	 */
	protected void setFunction(String function) {
		this.function = function;
	}

	/**
	 * @return Returns the function.
	 */
	protected String getFunction() {
		return function;
	}

	/**
	 * @param rewriteid The rewriteid to set.
	 */
	protected void setRewriteid(boolean rewriteid) {
		this.rewriteid = rewriteid;
	}

	/**
	 * @return Returns the rewriteid.
	 */
	protected boolean isRewriteid() {
		return rewriteid;
	}

	/**
	 * @param attributesNames The attributesNames to set.
	 */
	protected void setAttributesNames(String attributesNames) {
		this.attributesNames = attributesNames;
	}

	/**
	 * @return Returns the attributesNames.
	 */
	protected String getAttributesNames() {
		return attributesNames;
	}
}
