package org.ajax4jsf.framework.ajax.xmlfilter.tidy;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;

import org.ajax4jsf.framework.ajax.xmlfilter.BaseXMLFilter;
import org.ajax4jsf.framework.ajax.xmlfilter.FilterServletResponseWrapper;
import org.ajax4jsf.framework.ajax.xmlfilter.HtmlParser;
import org.ajax4jsf.framework.ajax.xmlfilter.TidyServletResponseWrapper;
import org.ajax4jsf.framework.util.message.Messages;
import org.ajax4jsf.io.parser.FastHtmlParser;
import org.apache.commons.collections.ArrayStack;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TidyXMLFilter extends BaseXMLFilter {
	static final Log log = LogFactory.getLog(TidyXMLFilter.class);
	private Properties _tidyProperties;
	private static final int STACK_SIZE = 100;

	/**
	 * Stack for handle parsers pool - to avoid instantiane on each request.
	 */
	private ArrayStack _parsersPool = new ArrayStack(STACK_SIZE);	

	public TidyXMLFilter() {}

	/**
     * Peturn parser to pool
	 * @param parser
	 */
	protected void reuseParser(HtmlParser parser) {
//		synchronized (_parsersPool) {
//			if (_parsersPool.size()<STACK_SIZE) {
//				if (log.isDebugEnabled()) {
//					log.debug("Push JTidy parser to pool");
//				}
//				_parsersPool.push(parser);
//			}
//		}		
	}

    /**
     * Factory method for create parsing object - contain chain of parsing,
     * transformation and serialization of response output
     * @param string Encodings for parser
     * @return
	 */
	protected HtmlParser getParser(String mime, boolean isAjax) {
		// For non-ajax, parse only html types !
			if (isAjax || mime.startsWith("text/html") || mime.startsWith("application/xhtml+xml")) {
				if(isAjax || isForcexml()){
				TidyParser parser ;
				parser = new TidyParser(getTidyProperties());
				// If tidy not handle all requests, disable reorganising of html
				parser.setMoveElements(isForcexml());
				// Setup configuration.
				if (parser.setMime(mime)) {
					//			parser.setEncoding(encoding);
					if (log.isDebugEnabled()) {
						log.debug(Messages.getMessage(Messages.CREATE_JTIDY_INFO));
					}
					return parser;
				}
				} else {
					return new FastHtmlParser();
				}
			}
		return null;
	}
	
    private Properties getTidyProperties() {
		if (null == _tidyProperties) {
			_tidyProperties = new Properties();
			InputStream props = null;
			try {
				props = TidyXMLFilter.class.getResourceAsStream("tidy.properties");
				if (null != props) {
					_tidyProperties.load(props);
				}
				// Second part - user-defined properties.
				props = Thread.currentThread().getContextClassLoader().getResourceAsStream("tidy.properties");
				if (null != props) {
					_tidyProperties.load(props);
				}				
			} catch (IOException e) {
				// TODO Auto-generated catch block
				log.warn(Messages.getMessage(Messages.READING_TIDY_PROPERTIES_ERROR), e);
			} finally {
				if(null != props){
					try {
						props.close();
					} catch (IOException e) {
						// can be ignored
					}
				}
			}
		}
		return _tidyProperties;
		
	}


	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.ajax.xmlfilter.BaseFilter#getWrapper(javax.servlet.http.HttpServletResponse)
	 */
	protected FilterServletResponseWrapper getWrapper(HttpServletResponse response) throws ServletException {
		// TODO Auto-generated method stub
		return new TidyServletResponseWrapper(response);
	}	

}
