/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.renderer;

import org.ajax4jsf.framework.ajax.AjaxScript;
import org.ajax4jsf.framework.resource.InternetResource;


/**
 * Base class for all Ajax enabled components. Perform common task -
 * output javasript for component.
 * @author shura (latest modification by $Author: alexsmirnov $)
 * @version $Revision: 1.5 $ $Date: 2006/07/29 11:49:08 $
 *
 */
public abstract class AjaxComponentRendererBase extends HeaderResourcesRendererBase implements HeaderResourceProducer {

//	private InternetResource ajaxScript = getResource("/com/exadel/vcp/framework/ajax/scripts/AJAX.js");
	private static final String AJAX_SCRIPT = AjaxScript.class.getName();
	
	private InternetResource[] scripts = null;

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.renderer.HeaderResourceProducer#getHeaderScripts(javax.faces.context.FacesContext)
	 */
//	public LinkedHashSet getHeaderScripts(FacesContext context, UIComponent component) {
//		LinkedHashSet scripts = new LinkedHashSet() ; // Collections.singleton(ajaxScript.getUri(context, null));
//		scripts.add(ajaxScript.getUri(context, component));
//		String[] additionalScripts = getAdditionalScripts();
//		if (null != additionalScripts) {
//			for (int i = 0; i < additionalScripts.length; i++) {
//				String resource = additionalScripts[i];
//				scripts.add(getResource(resource).getUri(context, component));
//			}
//		}
//		return scripts;
//	}
	
	

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.renderer.HeaderResourcesRendererBase#getScripts()
	 */
	protected  InternetResource[] getScripts() {
		// Lazy create
		if (scripts == null) {
			InternetResource[] additionalScripts = getAdditionalScripts();
			// Custom scripts
			if (null != additionalScripts) {
				scripts = new InternetResource[additionalScripts.length+1];
				System.arraycopy(additionalScripts,0,scripts,1,additionalScripts.length);
			} else {
				scripts = new InternetResource[1];				
			}
			// Ajax script
			scripts[0] = getResource(AJAX_SCRIPT);			
		}

		return scripts;
	}



	/**
	 * Hoock method for append custom ajax scripts 
	 * @return
	 */
	protected InternetResource[] getAdditionalScripts() {
		return null;
	}


}
