/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.renderer;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import org.ajax4jsf.framework.ajax.AjaxViewRoot;
import org.ajax4jsf.framework.resource.InternetResource;

/**
 * @author asmirnov@exadel.com (latest modification by $Author: alexsmirnov $)
 * @version $Revision: 1.6 $ $Date: 2006/07/29 11:49:08 $
 * 
 */
public class AjaxViewRootRenderer extends AjaxContainerRenderer {


	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ajax4jsf.framework.renderer.AjaxComponentRendererBase#getScripts()
	 */
	protected InternetResource[] getScripts() {
		// TODO Auto-generated method stub
		return null;
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ajax4jsf.framework.renderer.RendererBase#doDecode(javax.faces.context.FacesContext,
	 *      javax.faces.component.UIComponent)
	 */
	protected void doDecode(FacesContext context, UIComponent component) {
		AjaxViewRoot root = (AjaxViewRoot) component;
		if(root.isAjaxRequest() && !root.isSubmitted()) {
			// Find submitted ajax region.
			String ajaxId = (String) context.getExternalContext().getRequestParameterMap().get(AJAX_PARAMETER_NAME);
			String ajaxRegion = root.getAjaxRegionId(ajaxId);
			if(null != ajaxRegion){
				root.setSubmittedRegion(ajaxRegion);
			}
		}
	}

	/*
	 * (non-Javadoc) For ajax requests, ViewRoot render childrens directly.
	 * 
	 * @see org.ajax4jsf.framework.renderer.AjaxContainerRenderer#getRendersChildren()
	 */
	public boolean getRendersChildren() {
		FacesContext context = FacesContext.getCurrentInstance();
		// For non Ajax request, view root not render children
		if(! AjaxRendererUtils.isAjaxRequest(context)) {
			return false;
		}
		// Also, if have page component as clild - delegate rendering of
		// children to it.
		if(isHavePageChild(context)) {
			return false;
		}
		// Ajax Request. Control all output.
		return true;
	}

	/**
	 * Special checks - have page component on viewRoot childrens. Danger - can
	 * produce illegal result for redirections.
	 * 
	 * @param context
	 * @return
	 */
	private boolean isHavePageChild(FacesContext context) {
		AjaxViewRoot root = (AjaxViewRoot) context.getViewRoot();
		return root.isHavePage();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ajax4jsf.framework.renderer.AjaxContainerRenderer#getComponentClass()
	 */
	protected Class getComponentClass() {
		return AjaxViewRoot.class;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ajax4jsf.framework.renderer.RendererBase#doEncodeEnd(javax.faces.context.ResponseWriter,
	 *      javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	protected void doEncodeEnd(ResponseWriter writer, FacesContext context, UIComponent component) throws IOException {
        UIComponent submittedComponent = ((AjaxViewRoot) component).getSubmittedRegion(context);
        if (null == submittedComponent) {
    		super.doEncodeEnd(writer,context,component);
		} else {
			submittedComponent.encodeEnd(context);
		}
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ajax4jsf.framework.renderer.RendererBase#doEncodeBegin(javax.faces.context.ResponseWriter,
	 *      javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	protected void doEncodeBegin(ResponseWriter writer, FacesContext context, UIComponent component) throws IOException {
		// Reset instance of submitted region
		AjaxViewRoot ajaxViewRoot = ((AjaxViewRoot) component);
		ajaxViewRoot.resetSubmittedRegionInstance();
// if(ajaxViewRoot.isAjaxRequest()){
// AjaxRendererUtils.loadBundles(context);
// }
        UIComponent submittedComponent = ajaxViewRoot.getSubmittedRegion(context);
        if (null == submittedComponent) {
    		super.doEncodeBegin(writer, context, component);
		} else {
			submittedComponent.encodeBegin(context);
		}
	}

}
