/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.renderer;

import java.util.Iterator;

import javax.faces.FactoryFinder;
import javax.faces.application.ApplicationFactory;
import javax.faces.context.FacesContext;
import javax.faces.render.RenderKit;
import javax.faces.render.RenderKitFactory;

import org.ajax4jsf.framework.util.message.Messages;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * @author shura (latest modification by $Author: slava_kabanovich $)
 * @version $Revision: 1.3 $ $Date: 2006/07/12 09:45:43 $
 *
 */
public class ChameleonRenderKitFactory extends RenderKitFactory {
    RenderKitFactory defaultFactory;
//    private static final String AJAX_BASE_RENDER_KIT_PARAMETER = "AJAX_BASE_RENDERKIT_ID";
//    private static final String AJAX_RENDER_KIT_PARAMETER = "AJAX_RENDER_KIT_ID";

    private static final Log _log = LogFactory.getLog(ChameleonRenderKitFactory.class);
    /**
     * @param defaultFactory
     */
    public ChameleonRenderKitFactory(RenderKitFactory defaultFactory) {
        this.defaultFactory = defaultFactory;
    }
    /**
     * @param renderKitId
     * @param renderKit
     */
    public void addRenderKit(String renderKitId, RenderKit renderKit) {
    	if (renderKit instanceof ChameleonRenderKit) {
//			ChameleonRenderKit chameleonRenderKit = (ChameleonRenderKit) renderKit;
//			chameleonRenderKit.setConfiguration(ConfigurationFactory.getRendererConfigurationInstance(renderKitId))	;
		}
        defaultFactory.addRenderKit(renderKitId, renderKit);
    }
    /**
     * @param context
     * @param renderKitId
     * @return
     */
    public RenderKit getRenderKit(FacesContext context, String renderKitId) {
        RenderKit renderKit = defaultFactory.getRenderKit(context, renderKitId);
        if (renderKit instanceof ChameleonRenderKit) {
        	if (_log.isDebugEnabled()) {
				_log.debug(Messages.getMessage(Messages.REQUEST_CHAMELEON_RENDER_KIT_INFO, renderKitId));
			}
        	String baseRenderKitId = null;
        	// TODO - get DefaultRenderKitId from ViewHandler ?
            try {
            	// IN JSF-RI verifications, context may be null !
            	if(null != context) {
				baseRenderKitId = context.getApplication()
						.getDefaultRenderKitId();
            	} else {
                	ApplicationFactory appFactory =(ApplicationFactory) FactoryFinder.getFactory(FactoryFinder.APPLICATION_FACTORY);
                	baseRenderKitId = appFactory.getApplication().getDefaultRenderKitId();
				}
			} catch (Exception e) {
				_log.warn(Messages.getMessage(Messages.GET_DEFAULT_RENDER_KIT_ERROR), e);
			}
            if(baseRenderKitId == null ){
                baseRenderKitId = RenderKitFactory.HTML_BASIC_RENDER_KIT;
            }
        	if (_log.isDebugEnabled()) {
				_log.debug(Messages.getMessage(Messages.DEFAULT_RENDER_KIT_INFO, baseRenderKitId));
			}
            ((ChameleonRenderKit) renderKit).setDefaultRenderKit(defaultFactory.getRenderKit(context, baseRenderKitId));
        }        
        return renderKit;
    }
    /**
     * @return
     */
    public Iterator getRenderKitIds() {
        return defaultFactory.getRenderKitIds();
    }
}
