/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.renderer;

import javax.faces.component.UIComponent;

/**
 * Resolve component variables
 * 
 * @author ayukhovich@exadel.com (latest modification by $Author:
 *         alexeyyukhovich $)
 * @version $Revision: 1.9 $ $Date: 2006/11/21 17:24:54 $
 */
public class ComponentsVariableResolver {
	private static String COMPONENTS_VARIABLE_RESOLVER = ComponentsVariableResolver.class
			.getName();

	private static String getAttributeName(RendererBase renderer) {
		return COMPONENTS_VARIABLE_RESOLVER + ":" + (renderer != null ? renderer.getClass().getName() : null);
	}
	
	public static ComponentVariables getVariables(RendererBase renderer,
			UIComponent component) {
		ComponentVariables variables;

		String attributeName = getAttributeName(renderer);
		
		variables = (ComponentVariables) component.getAttributes().get(attributeName);

		if (variables == null) {
			variables = new ComponentVariables();
		
			component.getAttributes().put(attributeName, variables);
		}

		return variables;
	}

	public static void removeVariables(RendererBase renderer,
			UIComponent component) {
		component.getAttributes().remove(getAttributeName(renderer));
	}
}
