/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.renderer.compiler;

import java.io.IOException;

import javax.faces.FacesException;
import javax.faces.component.UIComponent;

import org.ajax4jsf.framework.util.message.Messages;

/**
 * @author asmirnov@exadel.com (latest modification by $Author: slava_kabanovich $)
 * @version $Revision: 1.2 $ $Date: 2006/07/12 09:45:59 $
 *
 */
public class FacetElement extends ElementBase {

	public static final String CURRENT_FACET = "child_to_render";
	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.renderer.compiler.ElementBase#encode(org.ajax4jsf.framework.renderer.compiler.TemplateContext)
	 */
	public void encode(TemplateContext context) throws IOException {
		String facetName = getValue(context).toString();
		if (null == facetName) {
			throw new FacesException(Messages.getMessage(Messages.NO_FACET_NAME_ATTRIBUTE, getTag()));
		}
		UIComponent stored = (UIComponent) context
			.getParameter(CURRENT_FACET);
		// if inside childrens loop - render facet from current iteration component.
		// TODO - use cloned context
		UIComponent facet = null == stored?context.getComponent().getFacet(facetName):stored.getFacet(facetName);		
		if (null != facet && facet.isRendered()) {
			if (getChildren().size()>0) {
				// Store facet to render in <u:child> element.
				// TODO - use cloned context.
				context.putParameter(CURRENT_FACET, facet);
				// In fact, render all childrens.
				super.encode(context);
				if (null != stored) {
					context.putParameter(CURRENT_FACET, stored);
				} else {
					context.removeParameter(CURRENT_FACET);
				}
			} else {
				// empty element - render facet as-is.
				context.getRenderer().renderChild(context.getFacesContext(),facet);
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.renderer.compiler.ElementBase#encode(org.ajax4jsf.framework.renderer.compiler.TemplateContext, java.lang.String)
	 */
	public void encode(TemplateContext context, String breakPoint) throws IOException {
		// TODO Auto-generated method stub
		throw new FacesException(Messages.getMessage(Messages.BREAKPOINTS_UNSUPPORTED_ERROR, getValue(context)));
	}
	
	public void setName(Object name) {
		setValue(name);
	}

	public String getTag() {
		return HtmlCompiler.NS_UTIL_PREFIX+HtmlCompiler.FACET_TAG;
	}
	
	
}
