/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.renderer.compiler;

import java.io.IOException;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

import org.ajax4jsf.framework.renderer.RendererBase;
import org.xml.sax.SAXException;


/**
 * Public interface for compiled XML rendered template.
 * {@link javax.faces.render.Renderer} must simple call xml.encode(this,context,component); 
 * for produce {@link javax.faces.context.ResponseWriter} events.
 * @author asmirnov@exadel.com (latest modification by $Author: alexsmirnov $)
 * @version $Revision: 1.2 $ $Date: 2006/09/04 10:47:48 $
 *
 */
public interface PreparedTemplate {
	
	/**
	 * Encode this XML component. May be simple send element and attributes,
	 * call specified Renderer or RendererUtils method etc. Can encode children
	 * elements ( if exist ).
	 * @param renderer
	 * @param context
	 * @param component
	 * @throws IOException
	 */
	public void encode(RendererBase renderer, FacesContext context, UIComponent component) throws IOException;
	
	public void encode(TemplateContext context) throws IOException;

	public void encode(TemplateContext context,String breakPoint) throws IOException;
	/**
	 * @return List of childrens of this component.
	 */
	public List getChildren();
	
	/**
	 * Append child element to current.
	 * @param child
	 * @throws SAXException 
	 */
	public void addChild(PreparedTemplate child) throws SAXException;
	
	/**
	 * Set parent element in compiled tree for template.
	 * @param parent
	 * @throws SAXException 
	 */
	public void setParent(PreparedTemplate parent) throws SAXException;
	
	/**
	 * @return tag name for element.
	 */
	public String getTag();

	/**
	 * @return Returns the value. If is EL expression valueBinding - evaluate in.
	 */
	public Object getValue(TemplateContext context);

}
