/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.renderer.compiler;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.faces.FacesException;

import org.ajax4jsf.framework.renderer.RendererBase;
import org.ajax4jsf.framework.resource.InternetResource;
import org.ajax4jsf.framework.resource.InternetResourceBuilder;


/**
 * @author asmirnov@exadel.com (latest modification by $Author: maksimkaszynski $)
 * @version $Revision: 1.2 $ $Date: 2006/06/28 08:50:43 $
 *
 */
public class ResourceElement extends ElementBase {

	private Map attrs = new HashMap();
	/* (non-Javadoc)
	 * @see java.util.Map#put(java.lang.Object, java.lang.Object)
	 */
	public Object put(Object key, Object value) {
		return attrs.put(key, value);
	}

	/* (non-Javadoc)
	 * @see java.util.Map#get(java.lang.Object)
	 */
	public Object get(Object key) {
		return attrs.get(key);
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.renderer.compiler.RootElement#encode(javax.faces.render.Renderer, javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(TemplateContext context) throws IOException {
		InternetResource resource = findResource(context);
		if(null !=resource){
			resource.encodeBegin(context.getFacesContext(),context.getComponent(),attrs);
		}
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.renderer.compiler.RootElement#encode(javax.faces.render.Renderer, javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeEnd(TemplateContext context) throws IOException {
		InternetResource resource = findResource(context);
		if(null !=resource){
			resource.encodeEnd(context.getFacesContext(),context.getComponent());
		}
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.renderer.compiler.RootElement#getString(javax.faces.render.Renderer, javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public String getString(TemplateContext context) throws FacesException {
		InternetResource resource = findResource(context);
		if(null !=resource){
			return resource.getUri(context.getFacesContext(),context.getComponent());
		}
		return "";
	}

	private InternetResource findResource(TemplateContext context) {
		InternetResource resource;
		String path = (String) getValue(context);
		RendererBase renderer = context.getRenderer();
		if (renderer != null)  {
			resource = renderer.getResource(path);
		} else {
			resource = InternetResourceBuilder.getInstance().createResource(null, path);
		}
		
		return resource;
	}
	
	public String getTag() {
		return HtmlCompiler.NS_PREFIX+HtmlCompiler.RESOURCE_TAG;
	}
	
}
