/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.renderer.compiler;

import java.util.HashMap;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import org.ajax4jsf.framework.renderer.RendererBase;


/**
 * Incapsulate all current parameters for encoding template.
 * @author asmirnov@exadel.com (latest modification by $Author: alexeyyukhovich $)
 * @version $Revision: 1.2 $ $Date: 2006/05/22 09:54:32 $
 *
 */
public class TemplateContext {
	FacesContext facesContext;
	RendererBase renderer;
	UIComponent component;
	Map parameters = new HashMap();
	/**
	 * @param renderer
	 * @param facesContext
	 * @param component
	 */
	public TemplateContext(RendererBase renderer, FacesContext facesContext, UIComponent component) {
		// TODO Auto-generated constructor stub
		this.facesContext = facesContext;
		this.renderer = renderer;
		this.component = component;
	}
	/**
	 * @return Returns the component.
	 */
	public UIComponent getComponent() {
		return component;
	}
	/**
	 * @return Returns the facesContext.
	 */
	public FacesContext getFacesContext() {
		return facesContext;
	}
	/**
	 * @return Returns the renderer.
	 */
	public RendererBase getRenderer() {
		return renderer;
	}
	
	public void putParameter(Object key,Object value) {
		parameters.put(key,value);
	}
	
	public Object getParameter(Object key) {
		return parameters.get(key);
	}
	public void removeParameter(Object key) {
		parameters.remove(key);
		
	}
	public ResponseWriter getWriter() {
		return facesContext.getResponseWriter();
	}

	/**
	 * Create copy of this context for other child component.
	 * Used in iterations and facets for render template on current child
	 * @param child - component for template
	 * @return - new copy of component, with same parameters.
	 */
	public TemplateContext clone(UIComponent child){
		TemplateContext cloned = new TemplateContext(this.renderer,this.facesContext,child);
		cloned.parameters.putAll(this.parameters);
		return cloned;
	}
	
	/**
	 * Return a mutable Map representing the parameters associated wth this TemplateContext
	 */
	public Map getParameters() {
		return parameters; 
	}
}
