/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.resource;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.ajax4jsf.framework.ajax.xmlfilter.CacheContent;


/**
 * @author shura (latest modification by $Author: alexsmirnov $)
 * @version $Revision: 1.5 $ $Date: 2006/10/18 10:54:23 $
 *
 */
public class CachedResourceContext extends ResourceContext {
	private ResourceContext parent;
	private CacheContent content;

	/**
	 * @param parent
	 */
	public CachedResourceContext(ResourceContext parent) {
		this.setCacheEnabled(true);
		this.parent = parent;
		this.content = new CacheContent();
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#getOutputStream()
	 */
	public OutputStream getOutputStream() throws IOException {
		return content.getOutputStream(this.parent.getOutputStream());
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#getPathInfo()
	 */
	public String getPathInfo() {
		return this.parent.getPathInfo();
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#getQueryString()
	 */
	public String getQueryString() {
		return this.parent.getQueryString();
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#getRequestParameter(java.lang.String)
	 */
	public String getRequestParameter(String data_parameter) {
		return this.parent.getRequestParameter(data_parameter);
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#getResourceAsStream(java.lang.String)
	 */
	public InputStream getResourceAsStream(String path) {
		return this.parent.getResourceAsStream(path);
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#getSessionAttribute(java.lang.String)
	 */
	public Object getSessionAttribute(String name) {
		return this.parent.getSessionAttribute(name);
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#setDateHeader(java.lang.String, long)
	 */
	public void setDateHeader(String name, long value) {
		this.content.setDateHeader(name,value);
		this.parent.setDateHeader(name, value);
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#setHeader(java.lang.String, java.lang.String)
	 */
	public void setHeader(String name, String value) {
		this.content.setHeader(name,value);
		this.parent.setHeader(name, value);
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#setIntHeader(java.lang.String, int)
	 */
	public void setIntHeader(String name, int value) {
		this.content.setIntHeader(name,value);
		this.parent.setIntHeader(name, value);
	}

	/**
	 * @return Returns the content.
	 */
	CacheContent getContent() {
		return this.content;
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#getWriter()
	 */
	public PrintWriter getWriter() throws IOException {
		// TODO Auto-generated method stub
		return content.getWriter(this.parent.getWriter());
	}

	public void setContentType(String contentType) {
		this.parent.setContentType(contentType);
		this.content.setContentType(contentType);
	}

	public Object getResourceData() {
		// TODO Auto-generated method stub
		return parent.getResourceData();
	}

	public void setResourceData(Object codec) {
		parent.setResourceData(codec);
		
	}
	
	public String getInitParameter(String name) {
		return parent.getInitParameter(name);
	}

	public String getServletPath() {
		// TODO Auto-generated method stub
		return parent.getServletPath();
	}
	
	public void release() {
		super.release();
		parent.release();
	}
	
}
