/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.resource;

import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.IOException;

import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;


/**
 * @author asmirnov@exadel.com (latest modification by $Author: sergeysmirnov $)
 * @version $Revision: 1.1 $ $Date: 2006/04/28 02:03:19 $
 *
 */
public abstract class ImageRenderer extends BaseResourceRenderer {

	private static final String _tag = "img";
	private static final String _hrefAttr = "src";
	private static final String[][] _commonAttrs = {
			
	};
	/**
	 * @return Returns the commonAttrs.
	 */
	protected String[][] getCommonAttrs() {
		return null ; //_commonAttrs;
	}

	/**
	 * @return Returns the hrefAttr.
	 */
	protected String getHrefAttr() {
		return _hrefAttr;
	}

	/**
	 * @return Returns the tag.
	 */
	protected String getTag() {
		return _tag;
	}

	/**
	 * Send {@link BufferedImage} to response in concrete format (GIF,Jpeg,PNG)
	 * @param context - current faces context.
	 * @param image - imaje to send
	 */
	public abstract void sendImage(ResourceContext context,RenderedImage image) throws IOException;
	
	/**
	 * @return type of image for create {@link BufferedImage}
	 */
	public abstract int getImageType();
	
	/**
	 * Create blank {@link BufferedImage} for paint.
	 * Type of image determined by current renderer.
	 * @param width
	 * @param height
	 * @return new image
	 */
	public BufferedImage createImage(int width,int height){
		return new BufferedImage(width,height,getImageType());	
	}
	
	protected void customEncode(InternetResource resource, FacesContext context, Object data) throws IOException {
		if (resource instanceof Java2Dresource) {
			Java2Dresource j2d = (Java2Dresource) resource;
			Dimension dim = j2d.getDimensions(context, data);
			ResponseWriter writer = context.getResponseWriter();
			writer.writeAttribute("width", String.valueOf(dim.width), "width");
			writer.writeAttribute("height", String.valueOf(dim.height), "height");
		}
	}
}
