package org.ajax4jsf.framework.resource;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.IOException;
import java.io.InputStream;

import javax.faces.FacesException;
import javax.faces.context.FacesContext;

import org.ajax4jsf.framework.util.message.Messages;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */


/**
 * @author shura (latest modification by $Author: alexsmirnov $)
 * @version $Revision: 1.3 $ $Date: 2006/07/31 16:57:38 $
 *
 */
public class Java2Dresource extends InternetResourceBase {
	private static final Log log = LogFactory.getLog(Java2Dresource.class);
	
	/**
	 * Primary calculation of image dimensions - used when HTML code is generated
	 * to render IMG's width and height
	 * Subclasses should override this method to provide correct sizes of rendered images
	 * @param facesContext
	 * @return dimensions of the image to be displayed on page
	 */
	public Dimension getDimensions(FacesContext facesContext, Object data){
		return new Dimension(1,1);
	}
	/**
	 * Secondary calculation is used basically by {@link getImage} method
	 * @param resourceContext
	 * @return
	 */
	protected Dimension getDimensions(ResourceContext resourceContext){
		return new Dimension(1,1);
	}
	
	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.InternetResourceBase#getResourceAsStream(javax.faces.context.FacesContext, java.lang.Object)
	 */
	public InputStream getResourceAsStream(ResourceContext context) {
		// TODO Auto-generated method stub
		return super.getResourceAsStream(context);
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.InternetResourceBase#send(javax.faces.context.FacesContext, java.lang.Object)
	 */
	public void send(ResourceContext context) throws IOException {
		ImageRenderer renderer = (ImageRenderer) getRenderer();
		try {
			RenderedImage image = getImage(context);
			renderer.sendImage(context, image);
		} catch (Exception e) {
//			log.error(Messages.getMessage(Messages.SEND_IMAGE_ERROR, context.getPathInfo()), e);
			throw new FacesException(Messages.getMessage(Messages.SEND_IMAGE_ERROR_2), e);
		}
	}

	/**
	 * Create {@link RenderedImage} for send to client. can be used as Java2d or Java Advanced Imaging.
	 * @param context
	 * @return image to send.
	 */
	protected RenderedImage getImage(ResourceContext context){
		ImageRenderer renderer = (ImageRenderer) getRenderer();
		Dimension imageDimensions = getDimensions(context);
		BufferedImage image = renderer.createImage(imageDimensions.width,imageDimensions.height);
		Graphics2D graphics = image.createGraphics();
		// clear all image to transparent.
		//Commented on December 26 2005 by Maksim Kaszynski
/*		graphics.setComposite(AlphaComposite.Src);
		graphics.setBackground(new Color(0,0,0,0));//Transparent
		graphics.clearRect(0,0,getWidth(),getHeight());
*/		paint(context,graphics);
		graphics.dispose();
		return image;
	}

	/**
	 * Template method for create image as Applet-like paint. 
	 * @param context - current resource context.
	 * @param graphics2D - graphics to paint.
	 */
	protected void paint(ResourceContext context, Graphics2D graphics2D) {
		// TODO Auto-generated method stub
		
	}
}
