/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.resource;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;

/**
 * Abstraction context class for rendering resource ( image, script, style )
 * can be work in 2 variants - for simple ServletRequest and as JSF context encapsulation.
 * @author asmirnov@exadel.com (latest modification by $Author: alexsmirnov $)
 * @version $Revision: 1.6 $ $Date: 2006/10/18 10:54:23 $
 *
 */
public abstract class ResourceContext  {
	
	private boolean cacheEnabled = false;
	private Object resourceData;
	// response headers
	/**
	 * Delegate to {@link javax.servlet.ServletResponse} setHeader
	 * @param name name of header
	 * @param value new value
	 */
	public abstract void setHeader(String name, String value);
	
	/**
	 * Delegate to {@link javax.servlet.ServletResponse} setHeader
	 * @param name name of header
	 * @param value new value
	 */
	public abstract void setIntHeader(String name, int value);

	/**
	 * Delegate to {@link javax.servlet.ServletResponse} setHeader
	 * @param name name of header
	 * @param value new value
	 */
	public abstract void setDateHeader(String name, long value);
	
	/**
	 * @return
	 * @throws IOException 
	 */
	public abstract OutputStream getOutputStream() throws IOException;
	
	/**
	 * @return
	 */
	public abstract String getQueryString();
	
	/**
	 * @return
	 */
	public abstract String getPathInfo();
	
	/**
	 * @return
	 */
	public abstract String getServletPath();
	
	/**
	 * Get session attribute for given name. session not created
	 * @param name attribute name
	 * @return value for attribute, or null.
	 */
	public abstract Object getSessionAttribute(String name);

	public abstract InputStream getResourceAsStream(String path);

	/**
	 * Get request parameter for given name.
	 * @param data_parameter
	 * @return
	 */
	public abstract String getRequestParameter(String data_parameter);

	/**
	 * @return Returns the cacheEnabled.
	 */
	public boolean isCacheEnabled() {
		return this.cacheEnabled;
	}

	/**
	 * @param cacheEnabled The cacheEnabled to set.
	 */
	public void setCacheEnabled(boolean cacheEnabled) {
		this.cacheEnabled = cacheEnabled;
	}

	/**
	 * get output writer for send response.
	 * @return
	 */
	public abstract PrintWriter getWriter() throws IOException;

	/**
	 * Setup response content type as {@see javax.servlet.ServletResponse#setContentType(java.lang.String)}
	 * @param contentType
	 */
	public abstract void setContentType(String contentType);
	
	public abstract String getInitParameter(String name);

	public Object getResourceData() {
		// TODO Auto-generated method stub
		return resourceData;
	}

	public void setResourceData(Object data) {
		resourceData = data;
		
	}
	
	/**
	 * Release any data used by this context.
	 * Close buffers used by cached context, release FacesContext, if exist.
	 */
	public void release() {
		
	}
	
}
