/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.resource;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * @author asmirnov@exadel.com (latest modification by $Author: alexsmirnov $)
 * @version $Revision: 1.5 $ $Date: 2006/09/20 15:09:37 $
 *
 */
public class ServletResourceContext extends ResourceContext {
	
	private ServletContext context;
	private HttpServletRequest request;
	
	private HttpServletResponse response;

	/**
	 * @param request
	 * @param response
	 */
	public ServletResourceContext(ServletContext context, HttpServletRequest request, HttpServletResponse response) {
		this.context = context;
		this.request = request;
		this.response = response;
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#setHeader(java.lang.String, java.lang.String)
	 */
	public void setHeader(String name, String value) {
		response.setHeader(name,value);

	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#setIntHeader(java.lang.String, int)
	 */
	public void setIntHeader(String name, int value) {
		response.setIntHeader(name,value);

	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#setDateHeader(java.lang.String, long)
	 */
	public void setDateHeader(String name, long value) {
		response.setDateHeader(name,value);

	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#getOutputStream()
	 */
	public OutputStream getOutputStream() throws IOException {
		// TODO Auto-generated method stub
		return response.getOutputStream();
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#getQueryString()
	 */
	public String getQueryString() {
		return request.getQueryString();
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#getPathInfo()
	 */
	public String getPathInfo() {
		return request.getPathInfo();
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#getSessionAttribute(java.lang.String)
	 */
	public Object getSessionAttribute(String name) {
		return request.getSession(false).getAttribute(name);
	}

	public InputStream getResourceAsStream(String path) {
		// TODO Auto-generated method stub
		return context.getResourceAsStream(path);
	}

	public String getRequestParameter(String data_parameter) {
		// TODO Auto-generated method stub
		return request.getParameter(data_parameter);
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.ResourceContext#getWriter()
	 */
	public PrintWriter getWriter() throws IOException {
		// TODO Auto-generated method stub
		return response.getWriter();
	}

	public void setContentType(String contentType) {
		response.setContentType(contentType);		
	}

	public String getInitParameter(String name) {
		// TODO Auto-generated method stub
		return context.getInitParameter(name);
	}

	public String getServletPath() {
		// TODO Auto-generated method stub
		return request.getServletPath();
	}

}
