/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package org.ajax4jsf.framework.resource;

import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;

import javax.faces.FacesException;
import javax.faces.FactoryFinder;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.RenderKit;
import javax.faces.render.RenderKitFactory;

import org.ajax4jsf.framework.renderer.RendererBase;
import org.ajax4jsf.framework.renderer.compiler.HtmlCompiler;
import org.ajax4jsf.framework.renderer.compiler.PreparedTemplate;
import org.ajax4jsf.framework.renderer.compiler.TemplateContext;
import org.ajax4jsf.framework.skin.SkinFactory;

/**
 * @author shura
 *
 */
public class TemplateCSSRenderer extends StyleRenderer {
	
	private static final String COMPILED_TEMPLATE_PROPERTY = "compiled-template";

	private RendererBase renderer = new RendererBase() {

		protected Class getComponentClass() {
			// TODO Auto-generated method stub
			return UIComponent.class;
		}
		
	};
	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.BaseResourceRenderer#send(org.ajax4jsf.framework.resource.InternetResource, org.ajax4jsf.framework.resource.ResourceContext)
	 */
	public int send(InternetResource base, ResourceContext context) throws IOException {
		PreparedTemplate template = null;
		template = getTemplate(base, context);
		FacesContext facesContext = FacesContext.getCurrentInstance();
		if(null != facesContext) {
			// Create responseWriter.
			String defaultRenderKitId = facesContext.getApplication().getDefaultRenderKitId();
			if (null == defaultRenderKitId) {
				defaultRenderKitId = RenderKitFactory.HTML_BASIC_RENDER_KIT;
			}
			RenderKitFactory renderKitFactory = (RenderKitFactory) FactoryFinder.getFactory(FactoryFinder.RENDER_KIT_FACTORY);
			RenderKit renderKit = renderKitFactory.getRenderKit(facesContext,defaultRenderKitId);
			// TODO - handle response encoding 
			ResponseWriter responseWriter = renderKit.createResponseWriter(context.getWriter(),null,"UTF-8");
			facesContext.setResponseWriter(responseWriter);
			responseWriter.startDocument();
			// TODO - parameters and mock renderer/component ?
			// for first time, this template only allow skin or faces variables interaction
			template.encode(renderer,facesContext,null);
			responseWriter.endDocument();
			responseWriter.flush();
			responseWriter.close();
		} else {
			throw new FacesException("FacesContext for resource from template "+base.getKey()+" is null");
		}
		return 0;
	}

	/**
	 * @param base
	 * @param context
	 * @return
	 * @throws IOException
	 */
	public PreparedTemplate getTemplate(InternetResource base, ResourceContext context) throws IOException {
		PreparedTemplate template;
		//		if(base.isCacheable()){
					template = (PreparedTemplate) base.getProperty(COMPILED_TEMPLATE_PROPERTY);
		//		}
				if(null == template){
					HtmlCompiler compiler = new HtmlCompiler();
					InputStream resourceAsStream = base.getResourceAsStream(context);
					template = compiler.compile(resourceAsStream, base.getKey());
					base.setProperty(COMPILED_TEMPLATE_PROPERTY, template);
					resourceAsStream.close();
				}
		return template;
	}
	
	/* (non-Javadoc)
	 * @see org.ajax4jsf.framework.resource.BaseResourceRenderer#requireFacesContext()
	 */
	public boolean requireFacesContext() {
		return true;
	}

	public Object getData(InternetResource base, FacesContext context, Object data) {
		ResourceContext resourceContext = new FacesResourceContext(context);
		PreparedTemplate template;
		try {
			template = getTemplate(base, resourceContext);
		} catch (IOException e) {
			return null;
		}
		UIComponent component = (UIComponent) (data instanceof UIComponent ? data:null);
		TemplateContext templateContext = new TemplateContext(null,context,component);
		Object value = template.getValue(templateContext);
		if(null == value){
			int hashCode = SkinFactory.getInstance().getSkin(context).hashCode(context);
			value = ByteBuffer.allocate(4).putInt(hashCode).array();
		}
		return value;
	}

}
