/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.resource.cached;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.collections.LRUMap;

class DualLRUMap extends LRUMap {
	
	private Map reverseMap ;

	public DualLRUMap(int size) {
		super(size);
		reverseMap = new HashMap(size);
	}

	/* (non-Javadoc)
	 * @see org.apache.commons.collections.LRUMap#processRemovedLRU(java.lang.Object, java.lang.Object)
	 */
	protected void processRemovedLRU(Object key, Object value) {
		synchronized (this) {
		super.processRemovedLRU(key, value);
		reverseMap.remove(value);
		}
	}

	/* (non-Javadoc)
	 * @see org.apache.commons.collections.LRUMap#put(java.lang.Object, java.lang.Object)
	 */
	public Object put(Object key, Object value) {
		synchronized (this) {
			reverseMap.put(value, key);
			return super.put(key, value);				
		}
	}
	
	public Object getKey(Object value){
		synchronized (this) {
		Object key = reverseMap.get(value);
		if(!containsKey(key)){
			reverseMap.remove(value);
			key = null;
		}
		return key;
		}
	}

	/* (non-Javadoc)
	 * @see org.apache.commons.collections.LRUMap#get(java.lang.Object)
	 */
	public Object get(Object key) {
		synchronized (this) {
		return super.get(key);
		}
	}
	
}