/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.framework.taglib;

import javax.faces.component.StateHolder;
import javax.faces.context.FacesContext;
import javax.faces.el.EvaluationException;
import javax.faces.el.MethodBinding;
import javax.faces.el.MethodNotFoundException;

/**
 * @author shura (latest modification by $Author: sergeysmirnov $)
 * @version $Revision: 1.1 $ $Date: 2006/04/28 02:05:42 $
 *
 */
/**
 * Simple method binding for constant action outcome.
 * @author asmirnov@exadel.com (latest modification by $Author: sergeysmirnov $)
 * @version $Revision: 1.1 $ $Date: 2006/04/28 02:05:42 $
 *
 */
public class SimpleActionMethodBinding extends MethodBinding implements StateHolder {
	// private static final Log log =
	// LogFactory.getLog(SimpleActionMethodBinding.class);

	private String _outcome;

	public SimpleActionMethodBinding(String outcome) {
		_outcome = outcome;
	}

	public Object invoke(FacesContext facescontext, Object aobj[])
			throws EvaluationException, MethodNotFoundException {
		return _outcome;
	}

	public Class getType(FacesContext facescontext)
			throws MethodNotFoundException {
		return String.class;
	}

	// ~ StateHolder support
	// ----------------------------------------------------------------------------

	private boolean _transient = false;

	/**
	 * Empty constructor, so that new instances can be created when restoring
	 * state.
	 */
	public SimpleActionMethodBinding() {
		_outcome = null;
	}

	public Object saveState(FacesContext facescontext) {
		return _outcome;
	}

	public void restoreState(FacesContext facescontext, Object obj) {
		_outcome = (String) obj;
	}

	public boolean isTransient() {
		return _transient;
	}

	public void setTransient(boolean flag) {
		_transient = flag;
	}

	public String toString() {
		return _outcome;
	}
}
