/**
 * Copyright 2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ajax4jsf.framework.util.javascript;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

/**
 * @author shura (latest modification by $Author: sergeysmirnov $)
 * @version $Revision: 1.1 $ $Date: 2006/04/28 02:04:03 $
 *
 */
public class ScriptUtils
{

    /**
     * 
     */
    private ScriptUtils()
    {

    }

    public static String toScript(Object obj)
    {
        if (obj instanceof ScriptString)
        {
            return ((ScriptString) obj).toScript();
        }
        else if (obj instanceof Collection)
        {
            // Collections put as JavaScript array.
            Collection collection = (Collection) obj;
            StringBuffer ret = new StringBuffer("[");
            boolean first = true;
            for (Iterator iter = collection.iterator(); iter.hasNext();)
            {
                Object element = (Object) iter.next();
                if (!first)
                {
                    ret.append(',');
                }
                ret.append(toScript(element));
                first = false;
            }
            return ret.append(']').toString();
        }
        else if (obj instanceof Map)
        {
            // Maps put as JavaScript hash.
            Map map = (Map) obj;

            StringBuffer ret = new StringBuffer("{");
            boolean first = true;
            for (Iterator iter = map.keySet().iterator(); iter.hasNext();)
            {
                Object key = (Object) iter.next();
                if (!first)
                {
                    ret.append(',');
                }
//                if (key instanceof String)
//                {
//                    ret.append((String) key);
//                }
//                else
//                {
                    ret.append(toScript(key));
//                }
                ret.append(':').append(toScript(map.get(key)));
                first = false;
            }
            return ret.append('}').toString();
        }
        else if (obj instanceof Number || obj instanceof Boolean)
        {
            // numbers and boolean put as-is, without conversion
            return obj.toString();
        }
        else if(null==obj){
        	return "null";
        } else
        {
            // all other put as encoded strings.
            StringBuffer ret = new StringBuffer("\'");
            addEncoded(ret, obj);
            return ret.append('\'').toString();
        }
    }

    public static void addEncoded(StringBuffer buff, Object obj)
    {
        JSEncoder encoder = new JSEncoder();
        char chars[] = obj.toString().toCharArray();
        for (int i = 0; i < chars.length; i++)
        {
            char c = chars[i];
            if (!encoder.compile(c))
            {
                buff.append(encoder.encode(c));
            }
            else
            {
                buff.append(c);
            }
        }
    }
}
