/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package org.ajax4jsf.io.parser;

import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;
import java.util.Stack;

/**
 * Context to keep current parsing state.
 * @author shura
 *
 */
public class ParsingContext {
	
	private int _currentStatePosition=0;
	
	private StringBuffer buff = new StringBuffer(64);
	
	private ParserState baseState;
	
	private Stack parsersStack = new Stack();
	
	private OutputStream _stream;
	
	private Writer _writer;

    private KeywordState _lastMatched;

	
	/**
	 * @param stream
	 */
	public ParsingContext(OutputStream stream) {
		super();
		// TODO Auto-generated constructor stub
		_stream = stream;
	}

	/**
	 * @param writer
	 */
	public ParsingContext(Writer writer) {
		super();
		// TODO Auto-generated constructor stub
		_writer = writer;
	}

	/**
	 * @return Returns the baseState.
	 */
	public ParserState getBaseState() {
		return baseState;
	}

	/**
	 * @param baseState The baseState to set.
	 */
	public void setBaseState(ParserState baseState) {
		this.baseState = baseState;
	}
	
	public void pushState(ParserState baseState){
		parsersStack.push(this.baseState);
		this.baseState = baseState;
	}
	
	public ParserState popState(){
		baseState = (ParserState) parsersStack.pop();
		return baseState;
	}

	/**
	 * @return Returns the currentStatePosition.
	 */
	public int getCurrentStatePosition() {
		return _currentStatePosition;
	}
	/**
	 * @return Returns the currentStatePosition.
	 */
	public int incCurrentStatePosition() {
		return _currentStatePosition++;
	}

	/**
	 * @param currentStatePosition The currentStatePosition to set.
	 */
	public void setCurrentStatePosition(int currentStatePosition) {
		_currentStatePosition = currentStatePosition;
	}
	
	public void putChar(char c){
		buff.append(c);
	}
	
	private void sendBuffer(Writer out) throws IOException {
		out.write(buff.toString());
		buff.setLength(0);
	}

	private void sendBuffer(OutputStream out) throws IOException {
		for (int i = 0; i < buff.length(); i++) {
			out.write(buff.charAt(i));
		}
		buff.setLength(0);
	}
	
	public void send() throws IOException {
		if(null != _writer){
			sendBuffer(_writer);
		} else {
			sendBuffer(_stream);
		}
	}

	public void send(char c) throws IOException {
		if(buff.length()>0){
			send();
		}
		if(null != _writer){
			_writer.write(c);
		} else {
			_stream.write(c);
		}
	}

    public void setLastMatched(KeywordState state) {
		this._lastMatched = state;
		
	}

    /**
	 * @return Returns the _lastMatched.
	 */
	public KeywordState getLastMatched() {
		return _lastMatched;
	}
}
