/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package org.ajax4jsf.io.parser;

public class StringState extends ParserState {
	
	protected char[] string;
	
	private ParserState reachedState;
	
	StringState(String toCompare, ParserState reachedState){
		super();
		this.string = toCompare.toCharArray();
		this.reachedState = reachedState;
	}

	
	/* (non-Javadoc)
	 * @see org.ajax4jsf.io.parser.ParserState#getNextState(char, org.ajax4jsf.io.parser.ParsingContext)
	 */
	ParserState getNextState(char c, ParsingContext context) {
		if (context.getCurrentStatePosition()==string.length) {
			// String matched - switch to reached state
			context.setCurrentStatePosition(0);
			return this.reachedState.getNextState(c,context);
		} else if(isAcceptChar(c,context)){
			// Continue reaching.
			return this;
		}
		return super.getNextState(c, context);
	}


	boolean isAcceptChar(char c, ParsingContext context) {
		boolean matched = false;
		int currentStatePosition = context.getCurrentStatePosition();
		if(currentStatePosition<string.length){
			matched = c==string[currentStatePosition];
		}
		if(matched){
			context.incCurrentStatePosition(); 
		}
		else {
			context.setCurrentStatePosition(0);
		}
		return matched;
	}

}
