/**
 * Copyright 2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ajax4jsf.renderers.ajax;


import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import org.ajax4jsf.framework.ajax.AjaxActionComponent;
import org.ajax4jsf.framework.renderer.compiler.HtmlCompiler;
import org.ajax4jsf.framework.renderer.compiler.PreparedTemplate;


/**
 * @author shura
 * Ajax version of commandButton. Main differences with base <code>HtmlCommandButton</code> :
 * 1. Render html &lt;button&gt; element instead of &lt;input&gt; Since, may contains text, images etc.
 * 2. By default type "button" instead of "submit" 
 * 3. Images rendered inside of &lt;button&gt; element , input "image" type not supported. 
 */
public class AjaxCommandButtonRenderer extends AjaxTemplatedCommandRendererBase {
    public static final String RENDERER_TYPE = "org.ajax4jsf.components.AjaxButtonRenderer";

    PreparedTemplate htmlTemplate = HtmlCompiler.compileResource(AjaxCommandButtonRenderer.class,"templates/button.xhtml");

    /*
     * (non-Javadoc)
     * 
     * @see javax.faces.render.Renderer#getRendersChildren()
     */
    public boolean getRendersChildren() {
        return true;
    }

    public void encodeTypeAndImage(FacesContext context, UIComponent uiComponent) throws IOException {
    	ResponseWriter writer = context.getResponseWriter();
    	String type = (String)uiComponent.getAttributes().get("type");
		String image =	(String)uiComponent.getAttributes().get("image");
		if(image != null){
			image = context.getApplication().getViewHandler().getResourceURL(context,image);
			image = context.getExternalContext().encodeResourceURL(image);
			writer.writeAttribute("type","image","image");
			writer.writeURIAttribute("src",image,"image");
			Object value;
			if (null == uiComponent.getAttributes().get("alt") &&  null != (value = uiComponent.getAttributes().get("value")) ) {
				writer.writeAttribute("alt",value,"value");
			}
		} else {
			if(null != type ){
			writer.writeAttribute("type",type.toLowerCase(),"type");
			} else {
				writer.writeAttribute("type","button","type");				
			}
		}
    }

    protected Class getComponentClass() {
		// TODO Auto-generated method stub
		return AjaxActionComponent.class;
	}

	/* (non-Javadoc)
	 * @see org.ajax4jsf.renderers.ajax.AjaxCommandRendererBase#getTemplate()
	 */
	protected PreparedTemplate getTemplate(FacesContext context) {
			return htmlTemplate;
	}

	protected void doEncodeBegin(ResponseWriter writer, FacesContext context, UIComponent component) throws IOException {
		super.doEncodeBegin(writer, context, component);
	}
}
