/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.renderers.ajax;

import java.io.IOException;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.ValueHolder;
import javax.faces.component.html.HtmlCommandButton;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;

import org.ajax4jsf.framework.ajax.AjaxEvent;
import org.ajax4jsf.framework.renderer.AjaxComponentRendererBase;
import org.ajax4jsf.framework.renderer.AjaxRendererUtils;


/**
 * @author asmirnov@exadel.com (latest modification by $Author: alexsmirnov $)
 * @version $Revision: 1.12 $ $Date: 2007/01/05 20:24:53 $
 *
 */
public abstract class AjaxCommandRendererBase extends AjaxComponentRendererBase {

	protected void doDecode(FacesContext facesContext, UIComponent uiComponent) {
	
	    //super.decode must not be called, because value is handled here
	    if ( isSubmitted(facesContext, uiComponent))
	    {
	    	Map attributes = uiComponent.getAttributes();
			Object type = attributes.get("type");
        	if (null == type || ! "reset".equalsIgnoreCase((String)type) ) {
				ActionEvent event;
				event = new ActionEvent(uiComponent);
				uiComponent.queueEvent(event); 
			} 
			uiComponent.queueEvent( new AjaxEvent(uiComponent));
//	        org.ajax4jsf.framework.ajax.AjaxRegionListener.addRegionsFromComponent(uiComponent, facesContext);
	    }
	}

	public String getOnClick(FacesContext context, UIComponent component) {
	        StringBuffer onClick;
			if ( !getUtils().isBooleanAttribute(component,"disabled")) {
				onClick = AjaxRendererUtils.buildOnClick(component, context);
				if (!"reset".equals(component.getAttributes().get("type"))) {
					onClick.append(";return false;");
				}
			} else {
				onClick = new StringBuffer("return false;");
			}
	        return onClick.toString();
		}

	public void encodeChildren(FacesContext context, UIComponent component) throws IOException {
	    renderChildren(context, component);
	}

	public Object getValue(UIComponent uiComponent) {
	    if (uiComponent instanceof ValueHolder) {
	        return ((ValueHolder) uiComponent).getValue();
	    }
	    return uiComponent.getAttributes().get("value");
	}

	public String getType(UIComponent uiComponent) {
	    String type;
	    if (uiComponent instanceof HtmlCommandButton) {
	        type = ((HtmlCommandButton) uiComponent).getType();
	    } else {
	        type = (String) uiComponent.getAttributes().get("type");
	    }
	    if (type == null) {
	        type = "button";
	    }
	    return type;
	}

	
	protected boolean isSubmitted(FacesContext facesContext, UIComponent uiComponent) {
		// Componet accept only ajax requests. 
		if ( !AjaxRendererUtils.isAjaxRequest(facesContext)) {
			return false;
		}
		if(getUtils().isBooleanAttribute(uiComponent,"disabled")){
			return false;
		}
	    String clientId = uiComponent.getClientId(facesContext);
	    Map paramMap = facesContext.getExternalContext().getRequestParameterMap();
	    Object  value = paramMap.get( clientId );
		return null != value;
	}

}
