/**
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.sun.com/cddl/
 *   
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.ajax4jsf.tag;

import java.io.IOException;

import javax.el.ELException;
import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;

import org.ajax4jsf.framework.renderer.AjaxPhaseListener;
import org.ajax4jsf.framework.util.message.Messages;

import com.sun.facelets.FaceletContext;
import com.sun.facelets.FaceletException;
import com.sun.facelets.tag.TagAttribute;
import com.sun.facelets.tag.TagAttributeException;
import com.sun.facelets.tag.TagConfig;
import com.sun.facelets.tag.TagHandler;

/**
 * @author shura
 *
 */
public class KeepAliveHandler extends TagHandler {

	private final TagAttribute beanName;

	private final TagAttribute ajaxOnly;
	/**
	 * @param config
	 */
	public KeepAliveHandler(TagConfig config) {
		super(config);
        this.beanName = this.getRequiredAttribute("beanName");
        if (beanName != null) {
            if (!beanName.isLiteral()) {
                throw new TagAttributeException(this.tag, this.beanName, Messages.getMessage(Messages.MUST_BE_LITERAL_ERROR));
            }
        }
        this.ajaxOnly = getAttribute("ajaxOnly");
	}

	/* (non-Javadoc)
	 * @see com.sun.facelets.FaceletHandler#apply(com.sun.facelets.FaceletContext, javax.faces.component.UIComponent)
	 */
	public void apply(FaceletContext ctx, UIComponent parent)
			throws IOException, FacesException, FaceletException, ELException {
        // Get bean instance from EL expression.
		String name = beanName.getValue();
		boolean isAjaxOnly = ajaxOnly!=null?ajaxOnly.getBoolean(ctx):false;
        FacesContext facesContext = ctx.getFacesContext();
        String beanExpression = "#{"+name+"}";
        ValueBinding valueBinding = facesContext.getApplication().createValueBinding(beanExpression);
        Object bean = valueBinding.getValue(facesContext);
        String beanAttributeName = isAjaxOnly?AjaxPhaseListener.AJAX_BEAN_PREFIX:AjaxPhaseListener.VIEW_BEAN_PREFIX+name;
		// Put bean instance to ViewRoot. 
        facesContext.getViewRoot().getAttributes().put(beanAttributeName, bean);
	}

}
