/**
 * Copyright 2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ajax4jsf.taglib.ajax;

import javax.faces.component.UIComponent;

import org.ajax4jsf.renderers.ajax.AjaxPageRenderer;


/**
 * @author shura
 *
 * Other variant of AJAX View - render full html page with root <html>,
 * <head> ( filled from "head" facet ) and <body> areas.
 * 
 */
public class AjaxPageTag extends AjaxRegionTag {
    
    private String onload;
    private String onunload;

    /* (non-Javadoc)
     * @see javax.faces.webapp.UIComponentTag#getRendererType()
     */
    public String getRendererType() {
        return AjaxPageRenderer.RENDERER_TYPE;
    }
    /* (non-Javadoc)
     * @see javax.servlet.jsp.tagext.Tag#release()
     */
    public void release() {
        this.onload = null;
        this.onunload = null;
        super.release();
    }
    /* (non-Javadoc)
     * @see javax.faces.webapp.UIComponentTag#setProperties(javax.faces.component.UIComponent)
     */
    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        setStringProperty(component, "onload", this.onload);
        setStringProperty(component, "onunload", this.onunload);
    }
    /**
     * @return Returns the onload.
     */
    public String getOnload() {
        return onload;
    }
    /**
     * @return Returns the onunload.
     */
    public String getOnunload() {
        return onunload;
    }
    /**
     * @param onload The onload to set.
     */
    public void setOnload(String onload) {
        this.onload = onload;
    }
    /**
     * @param onunload The onunload to set.
     */
    public void setOnunload(String onunload) {
        this.onunload = onunload;
    }
}
